/********************************************************************************
 * Copyright (c) 2024 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/

import { Action, Args, hasStringProp, Operation } from '@eclipse-glsp/client';

export interface SetPageVisibilityOperation extends Operation {
    kind: typeof SetPageVisibilityOperation.KIND;

    pageNodeId: string;

    hidden?: boolean;

    /**
     * Custom arguments.
     */
    args?: Args;
}

export namespace SetPageVisibilityOperation {
    export const KIND = 'SetPageVisibility';

    export function is(object: unknown): object is SetPageVisibilityOperation {
        return Action.hasKind(object, KIND) && hasStringProp(object, 'pageNodeId');
    }

    export function create(pageNodeId: string, options: { hidden?: boolean; args?: Args } = {}): SetPageVisibilityOperation {
        return {
            kind: KIND,
            isOperation: true,
            pageNodeId,
            ...options
        };
    }
}
