/********************************************************************************
 * Copyright (c) 2024 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/

import { Action, Args, hasStringProp, ResponseAction } from '@eclipse-glsp/client';

export interface TinaResponseAction extends ResponseAction {
    kind: typeof TinaResponseAction.KIND;

    tinaResponse: string;

    /**
     * Custom arguments.
     */
    args?: Args;
}

export namespace TinaResponseAction {
    export const KIND = 'TinaResponseAction';

    export function is(object: unknown): object is TinaResponseAction {
        return Action.hasKind(object, KIND) && hasStringProp(object, 'tinaResponse');
    }

    export function create(tinaResponse: string, options: { responseId?: string; args?: Args } = {}): TinaResponseAction {
        return {
            kind: KIND,
            tinaResponse: tinaResponse,
            responseId: '',
            ...options
        };
    }
}
