/********************************************************************************
 * Copyright (c) 2022 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied:
 * -- GNU General Public License, version 2 with the GNU Classpath Exception
 * which is available at https://www.gnu.org/software/classpath/license.html
 * -- MIT License which is available at https://opensource.org/license/mit.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR MIT
 ********************************************************************************/
import {
    // StructureCompartmentView,
    CircularNode,
    // moveFeature,
    // resizeFeature,
    // RectangularNode
    // GCompartmentView
    // GNode,
    CircularNodeView,
    configureActionHandler,
    configureDefaultModelElements,
    configureModelElement,
    connectableFeature,
    ConsoleLogger,
    ContainerConfiguration,
    DefaultTypes,
    editLabelFeature,
    GCompartment,
    GCompartmentView,
    GEdge,
    GEdgeView,
    // StructureCompartmentView,
    GLabel,
    GLabelView,
    GLSPProjectionView,
    GNode,
    hoverFeedbackFeature,
    initializeDiagramContainer,
    LogLevel,
    moveFeature,
    RectangularNode,
    RectangularNodeView,
    resizeFeature,
    RoundedCornerNodeView,
    selectFeature,
    TYPES
} from '@eclipse-glsp/client';
import { Container, ContainerModule } from 'inversify';
import '../css/diagram.css';
import { CopyToClipboardAction } from './Actions/CopyToClipboardAction';
import { CopyToClipboardHandler } from './Handlers/CopyToClipboardHandler';

import { ModelUpdateListener } from './Handlers/ModelUpdateListener';
import { PageNode, PNMLGraph, ServerPageNode, ServiceNode} from './model';
import { ActivePagesOverlay } from './ui-extensions/active-pages-overlay';
import { EditInputSignalMenu } from './ui-extensions/edit-input-signals';
import { EditTokensMenu } from './ui-extensions/edit-tokens';
import { SignalClauseMenu } from './ui-extensions/signal-clause-menu';
import { TinaMenu } from './ui-extensions/tina-menu';
import { DirectedEdgeView, InhibitorArcView, PageNodeView, ReadArcView, ReferenceEdgeView } from './views';

const taskListDiagramModule = new ContainerModule((bind, unbind, isBound, rebind) => {
    rebind(TYPES.ILogger).to(ConsoleLogger).inSingletonScope();
    rebind(TYPES.LogLevel).toConstantValue(LogLevel.warn);
    bind(ActivePagesOverlay).toSelf().inSingletonScope();
    bind(TYPES.IUIExtension).toService(ActivePagesOverlay);
    bind(TYPES.IDiagramStartup).toService(ActivePagesOverlay);

    const context = { bind, unbind, isBound, rebind };

    // rebind(GLSPActionDispatcher).to(MyActionDispatcher).inSingletonScope();
    configureActionHandler(context, CopyToClipboardAction.KIND, CopyToClipboardHandler);

    bind(ModelUpdateListener).toSelf().inSingletonScope();
    bind(TYPES.ISModelRootListener).toService(ModelUpdateListener);

    bind(TinaMenu).toSelf().inSingletonScope();
    bind(TYPES.IUIExtension).to(TinaMenu);

    bind(SignalClauseMenu).toSelf().inSingletonScope();
    bind(TYPES.IUIExtension).to(SignalClauseMenu);

    bind(EditInputSignalMenu).toSelf().inSingletonScope();
    bind(TYPES.IUIExtension).toService(EditInputSignalMenu);

    bind(EditTokensMenu).toSelf().inSingletonScope();
    bind(TYPES.IUIExtension).toService(EditTokensMenu);

    configureDefaultModelElements(context);

    configureModelElement(context, DefaultTypes.EDGE, GEdge, GEdgeView);
    configureModelElement(context, 'place', CircularNode, CircularNodeView);
    configureModelElement(context, 'transition', GNode, RectangularNodeView);
    configureModelElement(context, 'arc', GEdge, DirectedEdgeView);
    configureModelElement(context, 'comp:header', GCompartment, GCompartmentView);
    configureModelElement(context, 'pageNode', PageNode, PageNodeView, { enable: [resizeFeature], disable: [hoverFeedbackFeature] });
    configureModelElement(context, 'label', GLabel, GLabelView, { enable: [editLabelFeature, moveFeature] });
    configureModelElement(context, 'markingLabel', GLabel, GLabelView);
    configureModelElement(context, DefaultTypes.GRAPH, PNMLGraph, GLSPProjectionView);
    configureModelElement(context, 'page', GCompartment, GCompartmentView);
    configureModelElement(context, 'referencePlace', CircularNode, CircularNodeView);
    configureModelElement(context, 'referenceTransition', GNode, RectangularNodeView);
    configureModelElement(context, 'refLink', GEdge, ReferenceEdgeView);
    configureModelElement(context, 'readArc', GEdge, ReadArcView);
    configureModelElement(context, 'inhibitorArc', GEdge, InhibitorArcView);
    configureModelElement(context, 'labelNode', RectangularNode, RectangularNodeView, {
        disable: [moveFeature, hoverFeedbackFeature, selectFeature]
    });
    configureModelElement(context, 'labelComp', GCompartment, GCompartmentView);
    configureModelElement(context, 'arcWeightLabel', GLabel, GLabelView, { enable: [editLabelFeature] });
    configureModelElement(context, 'topicTransitionType', RectangularNode, RoundedCornerNodeView);
    configureModelElement(context, 'topicLink', GEdge, GEdgeView);
    configureModelElement(context, 'serviceTransitionType', ServiceNode, RoundedCornerNodeView, { enable: [connectableFeature] });
    configureModelElement(context, 'serviceChannel', RectangularNode, RoundedCornerNodeView, { disable: [moveFeature] });
    configureModelElement(context, 'serviceChannelLink', GEdge, GEdgeView);
    configureModelElement(context, 'serviceServerLink', GEdge, GEdgeView);
    configureModelElement(context, 'serverPrototype', ServerPageNode, PageNodeView, {
        enable: [resizeFeature],
        disable: [hoverFeedbackFeature]
    });
    configureModelElement(context, 'signalClauseLabel', GLabel, GLabelView);
    configureModelElement(context, 'edgeTextLabel', GLabel, GLabelView);
    configureModelElement(context, 'topicLinkLimitLabel', GLabel, GLabelView, { enable: [editLabelFeature] });

    configureModelElement(context, 'nodePageNode', PageNode, PageNodeView, { enable: [resizeFeature], disable: [hoverFeedbackFeature] })
});

export function initializeTasklistDiagramContainer(container: Container, ...containerConfiguration: ContainerConfiguration): Container {
    return initializeDiagramContainer(container, taskListDiagramModule, ...containerConfiguration);
}
