/********************************************************************************
 * Copyright (c) 2024 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
import { GLSPCommandHandler, GLSPContextMenu } from '@eclipse-glsp/theia-integration';
import { CommandContribution, CommandRegistry, MenuContribution, MenuModelRegistry } from '@theia/core';
import { ApplicationShell } from '@theia/core/lib/browser';
import { inject, injectable } from 'inversify';
import { isPlace, ResetMarkingsOperation} from 'tasklist-glsp';

export namespace ResetMarkingsCommand {
    export const RESET_MARKINGS = 'reset-markings-command';
}

@injectable()
export class ResetMarkingsCommandContribution implements CommandContribution {
    @inject(ApplicationShell) protected readonly shell: ApplicationShell;
    registerCommands(commands: CommandRegistry): void {
        commands.registerCommand(
            { id: ResetMarkingsCommand.RESET_MARKINGS, label: 'Reset Markings' },
            new GLSPCommandHandler(this.shell, {
                actions: context => [ResetMarkingsOperation.create()],
                isEnabled: context => !context.isReadonly && context.selectedElements.filter(isPlace).length === 1,
                isVisible: context => !context.isReadonly && context.selectedElements.filter(isPlace).length === 1
            })
        );
    }
}

@injectable()
export class ResetMarkingsMenuContribution implements MenuContribution {
    static readonly EDIT = GLSPContextMenu.MENU_PATH.concat('edit');
    registerMenus(menus: MenuModelRegistry): void {
        menus.registerMenuAction(ResetMarkingsMenuContribution.EDIT, { commandId: ResetMarkingsCommand.RESET_MARKINGS });
    }
}
