/********************************************************************************
 * Copyright (c) 2022 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * https://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied:
 * -- GNU General Public License, version 2 with the GNU Classpath Exception
 * which is available at https://www.gnu.org/software/classpath/license.html
 * -- MIT License which is available at https://opensource.org/license/mit.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR MIT
 ********************************************************************************/
package org.eclipse.glsp.example.javaemf;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.glsp.server.diagram.BaseDiagramConfiguration;
import org.eclipse.glsp.server.types.EdgeTypeHint;
import org.eclipse.glsp.server.types.ShapeTypeHint;

public class TaskListDiagramConfiguration extends BaseDiagramConfiguration {

   @Override
   public List<ShapeTypeHint> getShapeTypeHints() {
      return List.of(
          new ShapeTypeHint(PNMLModelTypes.TRANSITION, true, true, false, true),
          new ShapeTypeHint("compartment", true, true, true, true),
          new ShapeTypeHint(PNMLModelTypes.PAGE, true, true, true, true)

      );
   }

   @Override
   public List<EdgeTypeHint> getEdgeTypeHints() {
      List<EdgeTypeHint> edgeHints = new ArrayList<>();

      edgeHints.add(createDynamicEdgeTypeHint(PNMLModelTypes.ARC, List.of(
          PNMLModelTypes.TRANSITION,
          PNMLModelTypes.REF_TRANSITION,
          PNMLModelTypes.PLACE,
          PNMLModelTypes.REF_PLACE)));

      edgeHints.add(createDynamicEdgeTypeHint(PNMLModelTypes.REFERENCE, List.of(
          PNMLModelTypes.TRANSITION,
          PNMLModelTypes.REF_TRANSITION,
          PNMLModelTypes.PLACE,
          PNMLModelTypes.REF_PLACE)));

      edgeHints.add(createDynamicEdgeTypeHint(PNMLModelTypes.TOPIC_LINK,
          List.of(PNMLModelTypes.PLACE, PNMLModelTypes.TOPIC_TRANSITION)));

      edgeHints.add(createDynamicEdgeTypeHint(PNMLModelTypes.SERVICE_SERVER_LINK,
          List.of(PNMLModelTypes.PLACE, PNMLModelTypes.SERVICE_TRANSITION)));

      edgeHints.add(createDynamicEdgeTypeHint(PNMLModelTypes.SERVICE_CHANNEL_LINK,
          List.of(PNMLModelTypes.PLACE, PNMLModelTypes.SERVICE_CHANNEL)));

      EdgeTypeHint inhibitorHint = createDefaultEdgeTypeHint(PNMLModelTypes.INHIBITOR_ARC);
      inhibitorHint.setSourceElementTypeIds(List.of(PNMLModelTypes.PLACE, PNMLModelTypes.REF_PLACE));
      inhibitorHint.setTargetElementTypeIds(List.of(PNMLModelTypes.TRANSITION, PNMLModelTypes.REF_TRANSITION));
      edgeHints.add(inhibitorHint);

      EdgeTypeHint readHint = createDefaultEdgeTypeHint(PNMLModelTypes.READ_ARC);
      readHint.setSourceElementTypeIds(List.of(PNMLModelTypes.PLACE, PNMLModelTypes.REF_PLACE));
      readHint.setTargetElementTypeIds(List.of(PNMLModelTypes.TRANSITION, PNMLModelTypes.REF_TRANSITION));
      edgeHints.add(readHint);

      return edgeHints;
   }

   protected EdgeTypeHint createDynamicEdgeTypeHint (String type, List<String> connectableElements) {
      EdgeTypeHint edgeHint = createDefaultEdgeTypeHint(type);
      edgeHint.setDynamic(true);

      edgeHint.setSourceElementTypeIds(connectableElements);
      edgeHint.setTargetElementTypeIds(connectableElements);
      return edgeHint;
   }

}
