package org.eclipse.glsp.example.javaemf.handler.DiNeROS;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.glsp.example.javaemf.operation.AddDiNeROSTokenOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public class AddDiNeROSTokenHandler extends AbstractDiNeROSTokenHandler<AddDiNeROSTokenOperation> {

    @Override
    public Optional<Command> createCommand(AddDiNeROSTokenOperation operation) {
        Place place = modelState.getIndex().getEObject(operation.getPlaceId(), Place.class).orElseThrow(() ->
            new IllegalArgumentException("No place with this id exists"));

        //Uses a compound Command to potentially append the creation commands for toolInfo, balloonMarkings, tokens (if necessary)
        CompoundCommand command = new CompoundCommand();
        String balloonToken = operation.getToken();

        ToolInfoPlace toolInfoPlace = getOrCreateToolInfo(place, command);
        BalloonMarking balloonMarking = getOrCreateBalloonMarking(toolInfoPlace, command);
        Tokens tokens = getOrCreateTokens(balloonMarking, command);
        setInitialMarking(place, tokens.getToken().size() + 1, command);

        command.append(AddCommand.create(modelState.getEditingDomain(), tokens,
            PtnetPackage.Literals.TOKENS__TOKEN, balloonToken));
        return Optional.of(command);

    }

}






















