package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import java.util.Optional;

import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.handler.NodeCreation.CreatePageHandler;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.pnml.emf.ptnet.*;

public class CreateNodePageNodeHandler extends CreatePageHandler
{
    public CreateNodePageNodeHandler(){super(PNMLModelTypes.NODE_PAGE_NODE);}
 
    @Override
    public String getLabel(){return "NodePageNode";}

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation)
    {
        GModelElement container = 
            modelState.getIndex().get(operation.getContainerId())
                        .orElseGet(modelState::getRoot);

        EObject parentElement = getParentElement(container);
        EReference relationshipLiteral = getRelationshipLiteral(container);

        Page newPage = PtnetFactory.eINSTANCE.createPage();
        setElementId(newPage);
        setInitialName(newPage, "Node");
        newPage.getToolspecific().add(createNodePageToolInfo(newPage));

        NodeGraphics shape = 
            createShape(idGenerator.getOrCreateId(newPage),
            getRelativeLocation(operation), 
            GraphUtil.dimension(
                PNMLModelTypes.DefaultSizes.NodePageNodeWidth, 
                PNMLModelTypes.DefaultSizes.NodePageNodeHeight));

        newPage.setNodegraphics(shape);

        Command pageCommand = 
            AddCommand.create(
                modelState.getEditingDomain(), 
                parentElement,
                relationshipLiteral, 
                newPage);

        return Optional.of(pageCommand);
    }

    @Override
    protected EObject getParentElement(final GModelElement container)
    {
        // prevent node page as root page (should be normal page, not a node)
        if(container.getType().equals(PNMLModelTypes.GRAPH))
            throw new IllegalCallerException("Node Page must be placed inside a Page.");
        
        //only basic page my be container for node page
        if(!container.getType().equals(PNMLModelTypes.PAGE_NODE))
            throw new IllegalCallerException("Node Page should only be contained inside a (basic) Page.");

        // check if the new node page would be nested inside an existing node page (forbidden)
        GModelElement temp = container;
        while (temp != null)
        {
            String type = temp.getType();

            if(type.equals(PNMLModelTypes.GRAPH)) 
                break;

            if(type.equals(PNMLModelTypes.NODE_PAGE_NODE))
                throw new IllegalCallerException("Node Pages may not in any way contain a Node Page.");
            
            temp = temp.getParent();
        }
        return modelState.getIndex().getEObject(container.getId()).orElseThrow();
    }

    protected ToolInfoPage createNodePageToolInfo (Page newPage)
    {
        ToolInfoPage toolSpecifics = PtnetFactory.eINSTANCE.createToolInfoPage();
        toolSpecifics.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
        toolSpecifics.setVersion(PNMLModelTypes.DINEROS_VERSION);
        toolSpecifics.setType(PNMLModelTypes.NODE_PAGE_NODE);
        toolSpecifics.setServiceName(newPage.getName().getText());
        return toolSpecifics;
    }
}
