package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.handler.NodeCreation.CreatePageHandler;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public class CreateServerPageHandler extends CreatePageHandler {

    public CreateServerPageHandler() {
        super(PNMLModelTypes.SERVER_PAGE);
    }

    @Override
    public String getLabel() {
        return "ServerPage";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        GModelElement container = modelState.getIndex().get(operation.getContainerId()).orElseGet(modelState::getRoot);

        EObject parentElement = getParentElement(container);
        EReference relationshipLiteral = getRelationshipLiteral(container);

        Page newPage = PtnetFactory.eINSTANCE.createPage();
        setElementId(newPage);
        setInitialName(newPage, "Server");
        newPage.getToolspecific().add(createServerPageToolInfo(newPage));

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newPage),
            getRelativeLocation(operation), GraphUtil.dimension(PNMLModelTypes.DefaultSizes.ServerPageWidth, PNMLModelTypes.DefaultSizes.ServerPageHeight));
        newPage.setNodegraphics(shape);



        Command pageCommand = AddCommand.create(modelState.getEditingDomain(), parentElement,
            relationshipLiteral, newPage);


        return Optional.of(pageCommand);
    }

    protected ToolInfoPage createServerPageToolInfo (Page newPage){
        ToolInfoPage toolSpecifics = PtnetFactory.eINSTANCE.createToolInfoPage();
        toolSpecifics.setTool(PNMLModelTypes.DINEROS_TOOL_NAME);
        toolSpecifics.setVersion(PNMLModelTypes.DINEROS_VERSION);
        toolSpecifics.setType(PNMLModelTypes.SERVER_PAGE);
        toolSpecifics.setServiceName(newPage.getName().getText());
        return toolSpecifics;
    }

}
