package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.operations.CreateEdgeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Objects;
import java.util.Optional;

public class CreateServiceChannelLinkHandler extends EMFCreateOperationHandler<CreateEdgeOperation> {
    @Inject
    protected ModelAccessUtil modelAccessUtil;

    public CreateServiceChannelLinkHandler() {
        super(PNMLModelTypes.SERVICE_CHANNEL_LINK);
    }


    @Override
    public String getLabel() { return "ServiceChannelLink"; }

    @Override
    public Optional<Command> createCommand(CreateEdgeOperation createEdgeOperation) {
        EditingDomain editingDomain = modelState.getEditingDomain();

        String sourceId = createEdgeOperation.getSourceElementId();
        String targetId = createEdgeOperation.getTargetElementId();

        GModelElement sourceGraphElement = modelState.getIndex().get(sourceId).orElseThrow();
        GModelElement targetGraphElement = modelState.getIndex().get(targetId).orElseThrow();


        Optional<EObject> source = modelState.getIndex().getEObject(sourceId);
        Optional<EObject> target = modelState.getIndex().getEObject(targetId);

        if(PNMLModelTypes.SERVICE_CHANNEL.equals(sourceGraphElement.getType())
            && target.isPresent()
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(target.get().eClass()))
        {
            // source is a channel and target a place node
            // therefore the link was drawn from a channel to a place node
            // therefore a response link is created

            String serviceTransitionId = modelState.getIndex().get(sourceId).get().getParent().getParent().getId();
            Transition serviceTransition = (Transition) modelState.getIndex().getEObject(serviceTransitionId).orElseThrow();

            ToolInfoTransition toolSpecifics = modelAccessUtil.getDiNeROSTransitionToolSpecific(serviceTransition, PNMLModelTypes.SERVICE_TRANSITION).orElseThrow();

            Channel channel = toolSpecifics.getChannels().getChannel().stream().filter(channelElement ->
                Objects.equals(sourceId, channelElement.getCid())).findFirst().orElseThrow();

            return Optional.of(
                SetCommand.create(editingDomain, channel, PtnetPackage.Literals.CHANNEL__RESPONSE, targetId)
            );
        } else if (PNMLModelTypes.SERVICE_CHANNEL.equals(targetGraphElement.getType())
            && source.isPresent()
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(source.get().eClass()))
        {
            // target is a channel and source a place node
            // therefore the link was drawn from a place node to a channel
            // therefore a request link is created

            String serviceTransitionId = modelState.getIndex().get(targetId).get().getParent().getParent().getId();
            Transition serviceTransition = (Transition) modelState.getIndex().getEObject(serviceTransitionId).orElseThrow();

            ToolInfoTransition toolSpecifics = modelAccessUtil.getDiNeROSTransitionToolSpecific(serviceTransition, PNMLModelTypes.SERVICE_TRANSITION).orElseThrow();

            Channel channel = toolSpecifics.getChannels().getChannel().stream().filter(channelElement ->
                Objects.equals(targetId, channelElement.getCid())).findFirst().orElseThrow();

            return Optional.of(
                SetCommand.create(editingDomain, channel, PtnetPackage.Literals.CHANNEL__REQUEST, sourceId)
            );
        } else {
            throw new IllegalArgumentException("Channel link must be between channel and place node.");
        }


    }
}