package org.eclipse.glsp.example.javaemf.handler.DiNeROS.ElementCreation;


import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.model.ModelAccessUtil;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.operations.CreateEdgeOperation;
import org.pnml.emf.ptnet.PtnetPackage;
import org.pnml.emf.ptnet.ToolInfoTransition;
import org.pnml.emf.ptnet.Transition;

import java.util.Optional;

public class CreateServiceServerLinkHandler extends EMFCreateOperationHandler<CreateEdgeOperation> {
    @Inject
    ModelAccessUtil modelAccessUtil;

    public CreateServiceServerLinkHandler() {
        super(PNMLModelTypes.SERVICE_SERVER_LINK);
    }

    @Override
    public String getLabel() { return "ServiceChannelLink"; }

    @Override
    public Optional<Command> createCommand(CreateEdgeOperation createEdgeOperation) {
        EditingDomain editingDomain = modelState.getEditingDomain();

        String sourceId = createEdgeOperation.getSourceElementId();
        String targetId = createEdgeOperation.getTargetElementId();

        GModelElement sourceGraphElement = modelState.getIndex().get(sourceId).orElseThrow();
        GModelElement targetGraphElement = modelState.getIndex().get(targetId).orElseThrow();


        Optional<EObject> source = modelState.getIndex().getEObject(sourceId);
        Optional<EObject> target = modelState.getIndex().getEObject(targetId);

        if(PNMLModelTypes.SERVICE_TRANSITION.equals(sourceGraphElement.getType())
            && target.isPresent()
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(target.get().eClass()))
        {
            // source is a service node and target a place node
            // therefore the link was drawn from a service node to a place node
            // therefore an input / request link is created

            Transition serviceTransition = (Transition) modelState.getIndex().getEObject(sourceId).orElseThrow();

            ToolInfoTransition toolSpecifics = modelAccessUtil.getDiNeROSTransitionToolSpecific(serviceTransition, PNMLModelTypes.SERVICE_TRANSITION).orElseThrow();

            return Optional.of(
                SetCommand.create(editingDomain, toolSpecifics,
                    PtnetPackage.Literals.TOOL_INFO_TRANSITION__SERVER_INPUT, targetId)
            );
        } else if (PNMLModelTypes.SERVICE_TRANSITION.equals(targetGraphElement.getType())
            && source.isPresent()
            && PtnetPackage.eINSTANCE.getPlaceNode().isSuperTypeOf(source.get().eClass()))
        {
            // target is a service node and source a place node
            // therefore the link was drawn from a place node to a service node
            // therefore an output / response link is created

            Transition serviceTransition = (Transition) modelState.getIndex().getEObject(targetId).orElseThrow();

            ToolInfoTransition toolSpecifics = modelAccessUtil.getDiNeROSTransitionToolSpecific(serviceTransition, PNMLModelTypes.SERVICE_TRANSITION).orElseThrow();

            return Optional.of(
                SetCommand.create(editingDomain, toolSpecifics, PtnetPackage.Literals.TOOL_INFO_TRANSITION__SERVER_OUTPUT, sourceId)
            );
        } else {
            throw new IllegalArgumentException("Channel link must be between channel and place node.");
        }
    }
}
