package org.eclipse.glsp.example.javaemf.handler.EdgeAndRefCreation;

import com.google.inject.Inject;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.example.javaemf.validators.CyclicReferencesValidator;
import org.eclipse.glsp.server.emf.EMFCreateOperationHandler;
import org.eclipse.glsp.server.emf.EMFIdGenerator;
import org.eclipse.glsp.server.operations.CreateEdgeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;


public class CreateReferenceHandler extends EMFCreateOperationHandler<CreateEdgeOperation> {

    @Inject
    protected EMFIdGenerator idGenerator;

    @Inject
    protected CyclicReferencesValidator cyclicReferencesValidator;

    public CreateReferenceHandler() {
        super(PNMLModelTypes.REFERENCE);
    }

    @Override
    public String getLabel() { return "Reference"; }

    @Override
    public Optional<Command> createCommand(CreateEdgeOperation createEdgeOperation) {

        String sourceId = createEdgeOperation.getSourceElementId();
        String targetId = createEdgeOperation.getTargetElementId();

        Node source = (Node) modelState.getIndex().getEObject(sourceId).orElseThrow();
        Node target = (Node) modelState.getIndex().getEObject(targetId).orElseThrow();

        EReference featureLiteral;

        if(source.eClass().equals(PtnetPackage.Literals.REF_PLACE)){
            featureLiteral = PtnetPackage.Literals.REF_PLACE__REF;
            if(cyclicReferencesValidator.wouldBeCyclicReference((RefPlace) source, (PlaceNode) target)){
                //would be a cyclic reference
                return Optional.empty();
            }
        } else if (source.eClass().equals(PtnetPackage.Literals.REF_TRANSITION)){
            featureLiteral = PtnetPackage.Literals.REF_TRANSITION__REF;
            if(cyclicReferencesValidator.wouldBeCyclicReference((RefTransition) source, (TransitionNode) target)){
                //would be a cyclic reference
                return Optional.empty();
            }
        } else {
            return Optional.empty();
        }



        return Optional.of(createReference(source, target, featureLiteral));
    }


    protected Command createReference(Node source, Node target, EReference featureLiteral) {
        EditingDomain editingDomain = modelState.getEditingDomain();

        return SetCommand.create(editingDomain, source,
            featureLiteral, target);
    }

}

