package org.eclipse.glsp.example.javaemf.handler.NodeCreation;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;

public class CreatePageHandler extends AbstractCreateNodeHandler<CreateNodeOperation, Page> {

    public CreatePageHandler() {
        super(PNMLModelTypes.PAGE_NODE);
    }

    public CreatePageHandler(String type) {
        super(type);
    }

    @Override
    public String getLabel() {
        return "PageNode";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {

        GModelElement container = 
            modelState.getIndex().get(operation.getContainerId())
                .orElseGet(modelState::getRoot);

        denySecondRootPage(container);

        EObject parentElement = getParentElement(container);
        EReference relationshipLiteral = getRelationshipLiteral(container);

        Page newPage = PtnetFactory.eINSTANCE.createPage();
        initializeElement(newPage);

        NodeGraphics shape = 
            createShape(
                idGenerator.getOrCreateId(newPage),
                getRelativeLocation(operation), 
                GraphUtil.dimension(
                    PNMLModelTypes.DefaultSizes.PageWidth, 
                    PNMLModelTypes.DefaultSizes.PageHeight)
                );

        newPage.setNodegraphics(shape);

        Command pageCommand = 
            AddCommand.create( 
                modelState.getEditingDomain(), 
                parentElement,
                relationshipLiteral, 
                newPage);

        return Optional.of(pageCommand);
    }

    protected EObject getParentElement(final GModelElement container) 
    {
        String type = container.getType();

        //page is subpage of other page or node page
        if(type.equals(PNMLModelTypes.PAGE_NODE) || type.equals(PNMLModelTypes.NODE_PAGE_NODE))
            return modelState.getIndex().getEObject(container.getId()).orElseThrow();

        //page is top level page
        if(type.equals(PNMLModelTypes.GRAPH))
            return modelState.getSemanticModel(pnml.class).orElseThrow()
                .getNet().stream().findFirst().orElseThrow();
                    
        throw new IllegalArgumentException("Page must be placed in graph or other page.");
    }

    protected EReference getRelationshipLiteral(final GModelElement container) 
    {
        String type = container.getType();

        //page is subpage of other page or node page
        if(type.equals(PNMLModelTypes.PAGE_NODE) || type.equals(PNMLModelTypes.NODE_PAGE_NODE))
            return PtnetPackage.Literals.PAGE__PAGE;
        
        //page is top level page
        if(type.equals(PNMLModelTypes.GRAPH))
            return PtnetPackage.Literals.PETRI_NET__PAGE;

        throw new IllegalArgumentException("Page must be placed in graph or other page.");
    }

    protected void denySecondRootPage(final GModelElement container)
    {
        /* 
         * assuming all other elements ensure there is an enclosing page,
         * an empty model implies that no such page exists
         */
        if(!modelState.getRoot().eContents().isEmpty() && 
            container.getType().equals(PNMLModelTypes.GRAPH))

            throw new IllegalArgumentException(
                "One page has to encapsulate the model"+
                " - only one such page may exist at any time.");
    }
}
