package org.eclipse.glsp.example.javaemf.handler.NodeCreation;



import java.util.Optional;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GPoint;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;

import org.pnml.emf.ptnet.*;


public class CreateReferencePlaceHandler extends AbstractCreateNodeHandler<CreateNodeOperation, RefPlace> {

    public CreateReferencePlaceHandler() {
        super(PNMLModelTypes.REF_PLACE);
    }

    @Override
    public String getLabel() {
        return "Reference Place";
    }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        EditingDomain editingDomain = modelState.getEditingDomain();
        Optional<GPoint> relativeLocation = getRelativeLocation(operation);
        Page page = getContainerPage(operation);

        RefPlace newRefPlace = PtnetFactory.eINSTANCE.createRefPlace();
        initializeElement(newRefPlace);

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newRefPlace),
            relativeLocation, GraphUtil.dimension(70, 70));
        newRefPlace.setNodegraphics(shape);

        setDiNeROSRefPlaceInfo(newRefPlace, page.getId());

        Command refPlaceCommand = AddCommand.create(editingDomain, page,
            PtnetPackage.Literals.PAGE__REFERENCE_PLACE, newRefPlace);

        return Optional.of(refPlaceCommand);
    }

    protected void setDiNeROSRefPlaceInfo(RefPlace refplace, String pageNodeId){
        ToolInfoNode toolInfo = PtnetFactory.eINSTANCE.createToolInfoNode();
        setDiNeROSToolSpecifics(toolInfo, pageNodeId);
        refplace.getToolspecific().add(toolInfo);
    }
}