package org.eclipse.glsp.example.javaemf.handler.NodeCreation;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.util.GraphUtil;
import org.eclipse.glsp.server.operations.CreateNodeOperation;
import org.pnml.emf.ptnet.*;

import java.util.Optional;


public class CreateTransitionHandler extends AbstractCreateNodeHandler<CreateNodeOperation, Transition> {

    public CreateTransitionHandler() {
        super(PNMLModelTypes.TRANSITION);
    }

    @Override
    public String getLabel() { return "Transition"; }

    @Override
    public Optional<Command> createCommand(final CreateNodeOperation operation) {
        Page page = getContainerPage(operation);

        Transition newTransition = PtnetFactory.eINSTANCE.createTransition();
        initializeElement(newTransition);

        NodeGraphics shape = createShape(idGenerator.getOrCreateId(newTransition),
            getRelativeLocation(operation), GraphUtil.dimension(PNMLModelTypes.DefaultSizes.TransitionWidth, PNMLModelTypes.DefaultSizes.TransitionHeight));
        newTransition.setNodegraphics(shape);

        setDiNeROSTransitionInfo(newTransition, page.getId());


        Command transitionCommand = AddCommand.create(modelState.getEditingDomain(), page,
            PtnetPackage.Literals.PAGE__TRANSITION, newTransition);

        return Optional.of(transitionCommand);
    }

    protected void setDiNeROSTransitionInfo(Transition transition, String pageNodeId){
        ToolInfoTransition toolInfo = PtnetFactory.eINSTANCE.createToolInfoTransition();
        setDiNeROSToolSpecifics(toolInfo, pageNodeId);
        toolInfo.setType(PNMLModelTypes.DISCRETE_TRANSITION);
        transition.getToolspecific().add(toolInfo);
    }
}

