package org.eclipse.glsp.example.javaemf.model;

import com.google.inject.Inject;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.graph.GModelElement;
import org.eclipse.glsp.graph.util.RootAdapterUtil;
import org.eclipse.glsp.server.actions.ActionDispatcher;
import org.eclipse.glsp.server.emf.EMFIdGenerator;
import org.eclipse.glsp.server.emf.notation.EMFNotationModelIndex;
import org.eclipse.glsp.server.emf.notation.EMFSemanticIdConverter;
import org.pnml.emf.ptnet.Channels;
import org.pnml.emf.ptnet.ToolInfoTransition;
import org.pnml.emf.ptnet.Transition;

import java.util.*;
import java.util.function.Function;

public class ExtendedIndex extends EMFNotationModelIndex {
    @Inject
    protected ModelAccessUtil modelAccessUtil;

    //private Map<EClass, Set<EObject>> pnmlTypeToGElements = new HashMap<>();


    protected ExtendedIndex(EObject target, EMFSemanticIdConverter idConverter) {
        super(target, idConverter);
    }

    public void indexSemanticModel(EObject semanticElement) {

        TreeIterator<EObject> modelIterator = semanticElement.eResource().getAllContents();

        while (modelIterator.hasNext()) {
            EObject element = modelIterator.next();
            // adds service channels to the index
            if(element instanceof Transition && isServiceTransition((Transition) element)){
                indexServiceChannels((Transition) element);
            }
            String id = getOrCreateId(element);

            if (id != null){
                eObjectIndex.putIfAbsent(id, element);
            }
        }
    }

    protected void indexServiceChannels(Transition transition){
        Optional<ToolInfoTransition> toolSpecifics = transition.getToolspecific().stream()
            .filter(toolInfo -> Objects.equals(toolInfo.getType(), PNMLModelTypes.SERVICE_TRANSITION)).findFirst();
        toolSpecifics.map(ToolInfoTransition::getChannels).map(Channels::getChannel).ifPresent(channels -> {
            channels.forEach(channel ->
                eObjectIndex.putIfAbsent(channel.getCid(), channel)
            );
        });
    }

    public boolean isServiceTransition(Transition transition) {
        if(transition.getToolspecific() == null){
            return false;
        }
        return transition.getToolspecific().stream()
            .anyMatch(toolInfo -> Objects.equals(toolInfo.getType(), PNMLModelTypes.SERVICE_TRANSITION));
    }

    public static EMFNotationModelIndex getOrCreate(final GModelElement element,
                                                    final EMFSemanticIdConverter idGenerator) {
        return RootAdapterUtil.getOrCreate(element, root -> new ExtendedIndex(root, idGenerator),
            EMFNotationModelIndex.class);
    }

}
