/********************************************************************************
 * Copyright (c) 2022 EclipseSource and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * https://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied:
 * -- GNU General Public License, version 2 with the GNU Classpath Exception
 * which is available at https://www.gnu.org/software/classpath/license.html
 * -- MIT License which is available at https://opensource.org/license/mit.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR MIT
 ********************************************************************************/
package org.eclipse.glsp.example.javaemf.palette;

import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;

import org.eclipse.glsp.example.javaemf.PNMLModelTypes;
import org.eclipse.glsp.server.actions.TriggerEdgeCreationAction;
import org.eclipse.glsp.server.actions.TriggerNodeCreationAction;
import org.eclipse.glsp.server.features.toolpalette.PaletteItem;
import org.eclipse.glsp.server.features.toolpalette.ToolPaletteItemProvider;

public class TaskListToolPaletteItemProvider implements ToolPaletteItemProvider {

    @Override
    public List<PaletteItem> getItems(Map<String, String> args) {
        return Lists.newArrayList(nodes(), edges(), diNeROSElements());
    }

    private PaletteItem nodes() {
        PaletteItem createPlace = node(PNMLModelTypes.PLACE, "Place");
        PaletteItem createTransition = node(PNMLModelTypes.TRANSITION, "Transition");
        PaletteItem createPage = node(PNMLModelTypes.PAGE_NODE, "Page");
        PaletteItem createRefPlace = node(PNMLModelTypes.REF_PLACE, "Reference Place");
        PaletteItem createRefTransition = node(PNMLModelTypes.REF_TRANSITION, "Reference Transition");


        List<PaletteItem> nodes = Lists.newArrayList(
            createPlace,
            createTransition,
            createPage,
            createRefPlace,
            createRefTransition
        );
        return PaletteItem.createPaletteGroup("nodes", "Nodes", nodes, "symbol-property");
    }

    private PaletteItem edges() {
        PaletteItem createArc = edge(PNMLModelTypes.ARC, "Arc");
        PaletteItem createRef = edge(PNMLModelTypes.REFERENCE, "Reference");
        PaletteItem createInhibitorArc = edge(PNMLModelTypes.INHIBITOR_ARC, "Inhibitor Arc");
        PaletteItem createReadArc = edge(PNMLModelTypes.READ_ARC, "Read Arc");
        List<PaletteItem> transitions = Lists.newArrayList(
            createArc,
            createRef,
            createInhibitorArc,
            createReadArc
        );
        return PaletteItem.createPaletteGroup("edges", "Edges", transitions, "symbol-property");
    }

    private PaletteItem diNeROSElements() {
        PaletteItem createTopic = node(PNMLModelTypes.TOPIC_TRANSITION, "Topic Node");
        PaletteItem createService = node(PNMLModelTypes.SERVICE_TRANSITION, "Service Node");
        PaletteItem createServiceChannel = node(PNMLModelTypes.SERVICE_CHANNEL, "Service Channel");
        PaletteItem createServerPage = node(PNMLModelTypes.SERVER_PAGE, "Server Page");
        PaletteItem createTopicLink = edge(PNMLModelTypes.TOPIC_LINK, "Topic Link");
        PaletteItem createServiceChannelLink = edge(PNMLModelTypes.SERVICE_CHANNEL_LINK, "Service Link");
        PaletteItem createServiceServerlLink = edge(PNMLModelTypes.SERVICE_SERVER_LINK, "Server Link");

        PaletteItem createNodePageNode = node(PNMLModelTypes.NODE_PAGE_NODE, "Node Page");

        List<PaletteItem> nodes = Lists.newArrayList(
            createTopic,
            createService,
            createTopicLink,
            createServiceChannel,
            createServerPage,
            createServiceChannelLink,
            createServiceServerlLink,
            createNodePageNode
        );
        return PaletteItem.createPaletteGroup("dineros", "DiNeROS Elements", nodes, "symbol-property");
    }


    private PaletteItem node(String elementTypeId, String label) {
        return new PaletteItem(elementTypeId, label, new TriggerNodeCreationAction(elementTypeId));
    }

    private PaletteItem edge(String elementTypeId, String label) {
        return new PaletteItem(elementTypeId, label, new TriggerEdgeCreationAction(elementTypeId));
    }


}
